@@ -45,9 +45,9 @@ class ReconnectingWebsocket:
         self._is_binary = is_binary
         self._conn = None
         self._socket = None
-        self.ws = None
+        self.ws: Optional[ws.WebSocketClientProtocol] = None
         self.ws_state = WSListenerState.INITIALISING
-        self.reconnect_handle = None
+        self._queue = asyncio.Queue(loop=self._loop)
 
     async def __aenter__(self):
         await self.connect()
@@ -76,6 +76,7 @@ class ReconnectingWebsocket:
         self.ws_state = WSListenerState.STREAMING
         self._reconnects = 0
         await self._after_connect()
+        self._loop.call_soon(asyncio.create_task, self._read_loop())
 
     async def _before_connect(self):
         pass
@@ -95,56 +96,65 @@ class ReconnectingWebsocket:
             self._log.debug(f'error parsing evt json:{evt}')
             return None
 
-    async def recv(self):
-        res = None
-        while not res:
+    async def _read_loop(self):
+        while True:
+            res = None
             if not self.ws or self.ws_state != WSListenerState.STREAMING:
                 await self._wait_for_reconnect()
                 break
             if self.ws_state == WSListenerState.EXITING:
                 break
+            if self.ws.state == ws.protocol.State.CLOSING:
+                break
+            if self.ws.state == ws.protocol.State.CLOSED:
+                try:
+                    await self._reconnect()
+                except BinanceWebsocketUnableToConnect:
+                    return {
+                        'e': 'error',
+                        'm': 'Max reconnect retries reached'
+                    }
+                else:
+                    break
             try:
                 res = await asyncio.wait_for(self.ws.recv(), timeout=self.TIMEOUT)
             except asyncio.TimeoutError:
                 logging.debug(f"no message in {self.TIMEOUT} seconds")
             except asyncio.CancelledError as e:
                 logging.debug(f"cancelled error {e}")
+                break
             except asyncio.IncompleteReadError as e:
                 logging.debug(f"incomplete read error {e}")
-            except ws.ConnectionClosed as e:
-                logging.debug(f"connection closed {e}")
-                if self.ws_state != WSListenerState.EXITING:
-                    await self._wait_for_reconnect()
             except Exception as e:
                 logging.debug(f"exception {e}")
+                break
             else:
-                if self.ws_state == WSListenerState.EXITING:
+                if self.ws_state in (WSListenerState.EXITING, WSListenerState.RECONNECTING):
                     break
-                res = await self._try_handle_msg(res)
-                if self.ws_state == WSListenerState.EXITING:
+                res = self._handle_message(res)
+                if self.ws_state in (WSListenerState.EXITING, WSListenerState.RECONNECTING):
                     break
+
+            if res and self._queue.qsize() < 100:
+                await self._queue.put(res)
+
+    async def recv(self):
+        res = None
+        while not res:
+            try:
+                res = await asyncio.wait_for(self._queue.get(), timeout=self.TIMEOUT)
+            except asyncio.TimeoutError:
+                logging.debug(f"no message in {self.TIMEOUT} seconds")
         return res
 
     async def _wait_for_reconnect(self):
         while self.ws_state == WSListenerState.RECONNECTING:
             logging.debug("reconnecting waiting for connect")
-            await asyncio.sleep(0.01)
         if not self.ws:
             logging.debug("ignore message no ws")
         else:
             logging.debug(f"ignore message {self.ws_state}")
 
-    async def _try_handle_msg(self, res):
-        msg_res = self._handle_message(res)
-        if msg_res:
-            # cancel error timeout
-            if self.reconnect_handle:
-                self.reconnect_handle.cancel()
-            self.reconnect_handle = self._loop.call_later(
-                self.NO_MESSAGE_RECONNECT_TIMEOUT, self._no_message_received_reconnect
-            )
-        return msg_res
-
     def _get_reconnect_wait(self, attempts: int) -> int:
         expo = 2 ** attempts
         return round(random() * min(self.MAX_RECONNECT_SECONDS, expo - 1) + 1)
@@ -163,8 +173,6 @@ class ReconnectingWebsocket:
         if self.ws_state == WSListenerState.RECONNECTING:
             return
         self.ws_state = WSListenerState.RECONNECTING
-        if self.reconnect_handle:
-            self.reconnect_handle.cancel()
         await self.before_reconnect()
         if self._reconnects < self.MAX_RECONNECTS:
             reconnect_wait = self._get_reconnect_wait(self._reconnects)
@@ -581,6 +589,35 @@ class BinanceSocketManager:
         """
         return self._get_futures_socket(symbol.lower() + '@aggTrade', futures_type=futures_type)
 
+    def symbol_miniticker_socket(self, symbol: str):
+        """Start a websocket for a symbol's miniTicker data
+
+                https://github.com/binance/binance-spot-api-docs/blob/master/web-socket-streams.md#individual-symbol-mini-ticker-stream
+
+                :param symbol: required
+                :type symbol: str
+
+                :returns: connection key string if successful, False otherwise
+
+                Message Format
+
+                .. code-block:: python
+
+                    {
+                        "e": "24hrMiniTicker",  // Event type
+                        "E": 123456789,         // Event time
+                        "s": "BNBBTC",          // Symbol
+                        "c": "0.0025",          // Close price
+                        "o": "0.0010",          // Open price
+                        "h": "0.0025",          // High price
+                        "l": "0.0010",          // Low price
+                        "v": "10000",           // Total traded base asset volume
+                        "q": "18"               // Total traded quote asset volume
+                    }
+
+                """
+        return self._get_socket(symbol.lower() + '@miniTicker')
+
     def symbol_ticker_socket(self, symbol: str):
         """Start a websocket for a symbol's ticker data
 
@@ -1097,13 +1134,22 @@ class ThreadedWebsocketManager(ThreadedApiManager):
     ) -> str:
         return self._start_async_socket(
             callback=callback,
-            socket_name='aggtrade_socket',
+            socket_name='aggtrade_futures_socket',
             params={
                 'symbol': symbol,
                 'futures_type': futures_type,
             }
         )
 
+    def start_symbol_miniticker_socket(self, callback: Callable, symbol: str) -> str:
+        return self._start_async_socket(
+            callback=callback,
+            socket_name='symbol_miniticker_socket',
+            params={
+                'symbol': symbol,
+            }
+        )
+
     def start_symbol_ticker_socket(self, callback: Callable, symbol: str) -> str:
         return self._start_async_socket(
             callback=callback,
@@ -1227,7 +1273,7 @@ class ThreadedWebsocketManager(ThreadedApiManager):
     ) -> str:
         return self._start_async_socket(
             callback=callback,
-            socket_name='options_multiplex_socket',
+            socket_name='futures_multiplex_socket',
             params={
                 'streams': streams,
                 'futures_type': futures_type
